//Elevado (c) 2002 Ch. Gerber

#version unofficial MegaPov 0.7;

#include "colors.inc"
#include "mytextures.inc"

#declare HQ        = 1;
#declare Rad_on    = 1;
#declare Atmos_on  = 1;
#declare Clouds_on = 1;
#declare Forest_on = 1;
#declare Rock_on   = 1;
 
global_settings {
   assumed_gamma 2.6
   max_trace_level 50
   #if (Rad_on = 1)
    ini_option "+QR"
    radiosity {
      pretrace_start 0.08
      pretrace_end   0.01
      count 20
      nearest_count 5
      error_bound 0.4
      recursion_limit 1
      low_error_factor 0.5
      gray_threshold 0.75
      minimum_reuse 0.015
      brightness 1.25
      adc_bailout 0.006
      normal on
    }
   #end 
}


// --- Camera, Sun and Sky ---

#declare camLoc    = <0,25,-20>;
#declare camLookAt = <0,50,250>;
#declare lightDist = 1000000;
#declare lightPos  = vrotate (z*lightDist, <-45,-45,0>);
#declare groundRad = 25000; 

camera {
  location  camLoc
  direction z
  right 4/3*x
  look_at camLookAt
}

light_source {
  <0,0,0>
  color 1.5*<1.2,1.0,0.8>
  media_attenuation on
  translate lightPos
}

#if (Rad_on = 0)  //fill lights
light_source {
  <100,500,-1000>
  color rgb 0.5
  media_attenuation off 
  media_interaction off
  shadowless
}
light_source {
  <-1000,2000,-1000>
  color rgb 0.5
  media_attenuation off
  media_interaction off
}
#end

sky_sphere {
  pigment {
    planar
    color_map {
      [0.70  rgb <0.25,0.41,0.53>]
      [0.888 rgb <0.71,0.69,0.64>]
      [1     rgb <0.98,0.73,0.55>]
    }
    scale <1,2,1>
    rotate 5*x
    translate -0.125*y
  }
}

fog {fog_type 2 distance 200 rgb <0.98,0.73,0.55> fog_offset 0 fog_alt 20}   

#if (Atmos_on = 1) //atmosphere
sphere { 
  0, groundRad + 2000
  pigment {rgbf 1}
  interior {
    media {
      method 3
      intervals 2
      samples 4, 4
      scattering {2, rgb 0.0005*<0.04,0.06,0.1> extinction 1}
      density {
        spherical
        color_map {
	  [0.0 rgb 1.0]
	  [0.2 rgb 0.5]
	  [1.0 rgb 0.0]
        }
      }
    }
  }  
  translate -groundRad*y
  scale <1,0.5,1>
  hollow
}
#end  

#if (Clouds_on = 1)  //clouds   
#if (HQ=1) $ mIntervals = 3; $ mSamples = 6; #else $ mIntervals = 1; $ mSamples = 4; #end
difference {
  sphere {0, groundRad + 600}
  sphere {0, groundRad + 350}
  pigment {rgbf 1}
  interior {
    media {
      scattering {2, 0.4*<0.01,0.012,0.01> extinction 1}
      method 3
      intervals mIntervals
      samples mSamples, mSamples 
      density {
        wrinkles
        ramp_wave
        color_map {
          [0   rgb 0]
          [0.5 rgb 0]
          [0.7 rgb 0.4]
          [1   rgb 0.7]
        }
        scale <67,33,60>
        rotate y*25
        translate 85*y
      }
      density {
        marble
        triangle_wave
        turbulence 1.5
        color_map {
          [0    rgb 0]
          [0.4  rgb 0]
          [0.85 rgb 0.35]
          [1    rgb 0.7]
        }
        scale <1000,500,750>
        rotate -25*y
        translate 85*y
      }
    }
  } 
  translate -groundRad*y
  scale <1,0.5,1>
  hollow
}
#end    


//--- Landscape and Forest ---

#declare Land = height_field { 
  tga "land.tga" smooth translate -0.5*x
  scale <1000,60,1000>
  texture {
    slope y
    texture_map {
      [0.0  RockTex]
      [0.89 RockTex]
      [0.99 GroundTex]
      [1.0  GroundTex]
    }
  }
}

#declare Hills1 = height_field { 
  tga "hills1.tga" smooth translate -0.5*x
  scale <2000,140,2000>
  texture {
    slope y
    texture_map {
      [0.0  RockTex]
      [0.85 RockTex]
      [0.95 GroundTex]
      [1.0  GroundTex]
    }
  }
}

#declare Hills2 = height_field { 
  tga "hills2.tga" smooth translate -0.5*x
  scale <10000,650,10000>
  texture {GroundTex}
}

object {Land}
object {Hills1 translate < -400,0,1000>}
object {Hills2 translate <-3000,0,3000>}

#if (Forest_on = 1)

#include "oakmesh.inc"

#declare Oak01 = union {
  object {Oak_BranchesIteration1 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration2 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration3 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration4 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration5 texture {Oak_TrunkTexPlain}}
  object {Oak_BranchesIteration6 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_BranchesIteration7 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_Leaves uv_mapping  texture {Oak_LeafTexGreen} double_illuminate}
  scale 5
}

#declare Oak02 = union {
  object {Oak_BranchesIteration1 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration2 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration3 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration4 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration5 texture {Oak_TrunkTexPlain}}
  object {Oak_BranchesIteration6 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_BranchesIteration7 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_Leaves uv_mapping  texture {Oak_LeafTexBrown} double_illuminate}
  scale 5
}

#declare Oak03 = union {
  object {Oak_BranchesIteration1 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration2 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration3 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration4 uv_mapping texture {Oak_TrunkTex}}
  object {Oak_BranchesIteration5 texture {Oak_TrunkTexPlain}}
  object {Oak_BranchesIteration6 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_BranchesIteration7 texture {Oak_TrunkTexPlain} no_shadow}
  object {Oak_Leaves uv_mapping  texture {Oak_LeafTexRed} double_illuminate}
  scale 5
}

#macro SlopeTest(Norm,MaxSlope)
  #declare STest = false;
  #if (vlength(Norm)!=0)
    #declare STest = true;
    #local Test = acos(vdot(Norm,y)/(vlength(Norm)*vlength(y)))*(180/pi);
    #if (Test > MaxSlope) #if (Test < 180-MaxSlope) #declare STest = false; #end #end
  #end  
#end
   
#declare Norm = <0,0,0>;
#declare Xpos = -500;
#declare Zpos = 0;
#declare MaxSlope = 30;
#declare MaxHeight = 40;
#declare VegeSeed = seed(2002);
#declare Pig = pigment {spotted color_map {[0 rgb 1][1 rgb 0]} scale 10 translate <-50,0,250>}
#while (Xpos<500)
  #while (Zpos<1000)
    #declare SPos = <Xpos,100,Zpos>;
    #declare PP = eval_pigment(Pig,<Xpos,0,Zpos>);
    #declare XX = PP.x;
    #declare Inter = trace (Land,SPos,<0,-1,0>,Norm);
    SlopeTest(Norm,MaxSlope)
    #if (STest=true)
      #if (Inter.y<MaxHeight) 
        #switch (XX)
          #range (0.7,1.0) 
            object {
              Oak03
              scale 0.7+0.6*rand(VegeSeed)
              rotate 360*rand(VegeSeed)*y 
              translate <Inter.x+rand(VegeSeed)*5,Inter.y,Inter.z+rand(VegeSeed)*5> 
            }  
          #break
          #range (0.5,0.7)
            object {
              Oak02
              scale 0.7+0.6*rand(VegeSeed)
              rotate 360*rand(VegeSeed)*y 
              translate <Inter.x+rand(VegeSeed)*5,Inter.y,Inter.z+rand(VegeSeed)*5>
            }
          #break
          #else
            object {
              Oak01
              scale 0.7+0.6*rand(VegeSeed)
              rotate 360*rand(VegeSeed)*y 
              translate <Inter.x+rand(VegeSeed)*5,Inter.y,Inter.z+rand(VegeSeed)*5>
            }
        #end  
      #end 
    #end  
    #declare Zpos = Zpos + 10;
  #end           
  #declare Zpos = 0;
  #declare Xpos = Xpos + 10; 
#end

#end //Forest_on

                 
//--- Rock and Figures ---

#if (Rock_on = 1)

#macro StoneBlock (p1, p2, Bevel, BumpSize, BumpScale, CrackSize, CrackScale)
  #ifndef (BlockRandom) #local BlockRandom = seed(0); #end  
  object {  
    #local pt1 = p1;
    #local pt2 = p2;
    #if (pt1.x > pt2.x)
      #local tmp = pt1.x;
      #local pt1 = <pt2.x,pt1.y,pt1.z>;
      #local pt2 = <tmp,pt2.y,pt2.z>;
    #end 
    #if (pt1.y > pt2.y)
      #local tmp = pt1.y;
      #local pt1 = <pt1.x,pt2.y,pt1.z>;
      #local pt2 = <pt2.x,tmp,pt2.z>;
    #end 
    #if (pt1.z > pt2.z)
      #local tmp = pt1.x;
      #local pt1 = <pt1.x,pt1.y,pt2.z>; 
      #local pt2 = <pt2.x,pt2.y,tmp>; 
    #end                  
    #local dpt = <pt2.x-pt1.x,pt2.y-pt1.y,pt2.z-pt1.z>;
    #local bumpx = rand (BlockRandom)*100000;
    #local bumpy = rand (BlockRandom)*100000;
    #local bumpz = rand (BlockRandom)*100000;    
    #local ErosionOffset = 0;
    #local fnBrick  = function {"rounded_box",<Bevel,dpt.x/2+ErosionOffset,dpt.y/2+ErosionOffset,dpt.z/2+ErosionOffset>}
    #local fnBumps  = function {pigment {wrinkles turbulence 0.5 scale BumpScale}}
    #local fnCracks = function {pigment {wrinkles scale CrackScale}} 
    isosurface { 
      function {fnBrick(x,y,z)+fnBumps(x+bumpx,y+bumpy,z+bumpz)*BumpSize + fnCracks(x+bumpx,y+bumpy,z+bumpz)*CrackSize}
      method 2
      max_trace 5 
      eval
      contained_by {box {<-dpt.x/2,-dpt.y/2,-dpt.z/2>,<dpt.x/2,dpt.y/2,dpt.z/2>}} 
    }    
    translate <pt1.x+dpt.x/2,pt1.y+dpt.y/2,pt1.z+dpt.z/2>
  }  
#end 

#declare Rock = object {
  StoneBlock(<-7.5,-10,-7.5>,<7.5,10,7.5>,6,0,1,4,5) 
  texture {
    slope y
    texture_map {
      [0.0 StoneTex]
      [0.7 StoneTex]
      [1.0 GroundTex scale 0.01]
    }
  }
  scale 0.2 
}

object {Rock translate <2.9,24.2,-15>} 

#include "huntress.inc"
#declare Inter = trace (Rock,<-0.1,50,0>,<0,-1,0>);
object {Huntress scale <-1,1,1> rotate -45*y translate Inter translate <2.9,24.2,-15>}

#include "bird.inc"

#declare BoxMin = <-300, 40, 50>;
#declare BoxMax = < 300,210,500>;
#declare NBirds = 50;
#declare BirdSeed = seed(1);
#declare Count = 0;
#while (Count<NBirds)
  object {
    Bird scale 0.15
    rotate <-10+20*rand(BirdSeed),360*rand(BirdSeed),-10+20*rand(BirdSeed)>
    translate (BoxMax-BoxMin)*<rand(BirdSeed),rand(BirdSeed),rand(BirdSeed)>+BoxMin
  }
  #declare Count = Count + 1;
#end 
  
#end



